/*
 * $Id: VKCCategoryFavoritesSerializer.java,v 1.14 2006/03/27 22:08:14 marbobo Exp $
 * 
 * Copyright (c) Siemens AG 2008 
 *           (c) Universitt Paderborn 2008 
 * Licensed under EUPL 1.1. 
 * 
 * $Log: VKCCategoryFavoritesSerializer.java,v $
 * Revision 1.14  2006/03/27 22:08:14  marbobo
 * + Bugfixes und Refactoring
 *
 * Revision 1.13  2006/03/27 15:24:16  marbobo
 * + Bugfix
 *
 * Revision 1.12  2006/03/23 11:39:20  marbobo
 * + Mgliche NPE abgefangen
 *
 * Revision 1.11  2004/12/09 12:39:28  lawern
 * vergessene PreparedStatements geschlossen
 *
 * Revision 1.10  2004/12/01 21:44:15  marbobo
 * + Statement auf PreparedStatement umgestellt
 *
 * Revision 1.9  2004/11/24 13:01:11  lawern
 * Global "conn.setAutoCommit( false );" enfernt, weil berflssig.
 *
 * Revision 1.8  2004/11/24 09:25:52  lawern
 * Hibernate ist VKC-Geschichte!
 *
 * Revision 1.7  2004/11/18 15:55:06  jonyp
 * changed order of close-stmt to resultset/connect/session  in finally-blocks
 *
 * Revision 1.6  2004/10/21 23:49:17  marbobo
 * + CURRENT_TIMESTAMP ersetzt mit java-Anweisung
 *
 * Revision 1.5  2004/08/09 16:08:30  jonyp
 * uses new class VKCUserCategoryFavorites for retrieval
 *
 * Revision 1.4  2004/05/11 15:53:11  jonyp
 * added functions deleteCategoryFavorite
 *
 * Revision 1.3  2004/05/07 10:15:44  jonyp
 * added countDoc/Cat function + ask for maxCount during saveFavorites
 *
 * Revision 1.2  2004/05/06 15:09:24  jonyp
 * added parameter for maxCount of returned category favorites
 *
 * Revision 1.1  2004/05/05 15:02:44  vchris
 * Initiale Version
 *
 * 
 * Created on 27.04.2004
 */ 
package vkc.db.serializer;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.ArrayList;
import vkc.db.helper.DBHelper;
import vkc.exception.DBException;
import vkc.view.WebShare;
import vkc.db.entities.VKCUserCatFavorite;

/**
 * DOCUMENT ME!!!
 * 
 * @author vchris
 */
public class VKCCategoryFavoritesSerializer {
	public VKCCategoryFavoritesSerializer() {
	}
	
	/**
	 * Ldt die Kategorie-Favoriten fr den bergebenen User aus der Datenbank
	 *
	 * @param userId
	 */
	public final static ArrayList retrieveCategoryFavorites(Integer userId) throws DBException {
		if (userId == null) {
			return null;
		}

		Connection conn = null;

		try {
			conn = WebShare.database.getConnection();
			
			return retrieveCategoryFavorites(userId, conn);
		} catch (Exception e) {
			throw new DBException(e);
		} finally {
			DBHelper.close(conn);
		}
	}
	
	public final static ArrayList retrieveCategoryFavorites(Integer userId, Connection conn) throws DBException {
		if (userId == null || userId.intValue() <= 0) {
			return null;
		}
		
		PreparedStatement pStmt = null;
		ResultSet rs = null;
		ArrayList favoritesList = new ArrayList();
		
		try {
			pStmt = conn.prepareStatement("SELECT userId, categoryId, created FROM userCatFavorites WHERE userid = ? ORDER BY created DESC");
			pStmt.setInt(1, userId.intValue());
			rs = pStmt.executeQuery();

			while (rs.next()) {
				VKCUserCatFavorite ucf = new VKCUserCatFavorite();
			
				ucf.setUserId(new Integer(rs.getInt("userId")));
				ucf.setCategoryId(new Integer(rs.getInt("categoryId")));
				ucf.setCreated(rs.getTimestamp("created"));
			
				favoritesList.add(ucf);
			}

			return favoritesList;
		} catch (Exception e) {	
			throw new DBException(e);
		} finally {	
			DBHelper.close(rs);
			DBHelper.close(pStmt);
		}
	}

	/**
	 * Lscht die in der ArrayListe cats bergebenen Categorie-IDs aus der Tabelle
	 * USERCATFAVORITES fr den bergebenen User.
	 * 
	 * @param userId User, fr den die Favoriten gelscht werden sollen
	 * @param cats ArrayList mit Integer-Objekten der zu lschenen Favoriten
	 * @throws DBException
	 */
	public final static void deleteCategoryFavorites(Integer userId, ArrayList cats) throws DBException {
		Connection conn = null;

		try {
			conn = WebShare.database.getConnection();
			deleteCategoryFavorites(userId, cats, conn);
			conn.commit();
		} catch (Exception e) {
			DBHelper.rollBack(conn);
			throw new DBException(e);
		} finally {
			DBHelper.close(conn);
		}
	}
	
	/**
	 * Lscht Eintrge aus der Tabelle USERCATFAVORITES<br>
	 * <ul>
	 * <li>Werden die Parameter userId und cats angegeben, werden nur die Eintrge die in cats bergeben
	 *     wurden fr den bergebenen user gelscht
	 * <li>Wurde nur der Parameter userId bergeben (cats == null), werden alle Favoriten des Users gelscht
	 * <li>Wurde nur der Parameter cats bergeben (userId == null), werden alle Referenzen hierauf bei allen usern gelscht
	 * 
	 * @param userId
	 * @param cats
	 * @param conn
	 * @throws DBException
	 */
	public final static void deleteCategoryFavorites(Integer userId, ArrayList cats, Connection conn) throws DBException {
		if ((userId == null || userId.intValue() <= 0) && (cats == null || cats.isEmpty())) {
			return;
		}
		
		PreparedStatement pStmt = null;
		
		try {
			// Es wurden zu lschende Kategorien bergeben
			if (cats != null && !cats.isEmpty()) {
				if (userId != null && userId.intValue() > 0) {
					pStmt = conn.prepareStatement("DELETE FROM usercatfavorites WHERE userid = ? AND categoryid = ?");
					
					// Alle Referenzen auf die zu in der Arraylist bergebenen Kategorie sollen gelscht werden
					for (int i = 0; i < cats.size(); i++) {
						Integer categoryId = (Integer)cats.get(i);
					    
					    pStmt.setInt(1, userId.intValue());
						pStmt.setInt(2, categoryId.intValue());
						pStmt.execute();
					}
				} else {
					pStmt = conn.prepareStatement("DELETE FROM usercatfavorites WHERE categoryid = ?");
					
					// Alle Referenzen auf die zu in der Arraylist bergebenen Kategorie sollen gelscht werden
					for (int i = 0; i < cats.size(); i++) {
						Integer categoryId = (Integer)cats.get(i);
					    
						pStmt.setInt(1, categoryId.intValue());
						pStmt.execute();
					}
				}
			} else {
				// Es wurde ein zu lschender User bergeben
				pStmt = conn.prepareStatement("DELETE FROM usercatfavorites WHERE userid = ?");
				pStmt.setInt(1, userId.intValue());
			    pStmt.execute();
			}
		} catch (Exception e) {	
			throw new DBException(e);
		} finally {	
			DBHelper.close(pStmt);
		}
	}

	/**
	 * Lscht die in der ArrayListe cats bergebenen Categorie-IDs aus der Tabelle
	 * USERCATFAVORITES fr den bergebenen User.
	 * 
	 * @param userId User, fr den die Favoriten gelscht werden sollen
	 * @param cats ArrayList mit Integer-Objekten der zu lschenen Favoriten
	 * @throws DBException
	 */
	public final static void deleteCategoryFavorite(Integer userId, Integer categoryId) throws DBException {
		Connection conn = null;

		try {
			conn = WebShare.database.getConnection();
			deleteCategoryFavorite(userId, categoryId, conn);
			conn.commit();
		} catch (Exception e) {
			DBHelper.rollBack(conn);
			throw new DBException(e);
		} finally {
			DBHelper.close(conn);
		}
	}

	public final static void deleteCategoryFavorite(Integer userId, Integer categoryId, Connection conn) throws DBException {
		if ((userId == null || userId.intValue() <= 0) && (categoryId == null || categoryId.intValue() <= 0)) {
			return;
		}
	
		PreparedStatement pStmt = null;
	
		try {
            if (userId != null && categoryId == null) {  
				pStmt = conn.prepareStatement("DELETE FROM usercatfavorites WHERE userid = ?");
				pStmt.setInt(1, userId.intValue());
			    pStmt.execute();
            } else if (userId == null && categoryId != null) {  
				pStmt = conn.prepareStatement("DELETE FROM usercatfavorites WHERE categoryId = ?");
				pStmt.setInt(1, categoryId.intValue());
			    pStmt.execute();
			} else if (userId != null && categoryId != null) {  
				pStmt = conn.prepareStatement("DELETE FROM usercatfavorites WHERE categoryId = ? AND userId = ?");
				pStmt.setInt(1, categoryId.intValue());
				pStmt.setInt(2, userId.intValue());
			    pStmt.execute();
			}
		} catch (Exception e) {	
			throw new DBException(e);
		} finally {	
			DBHelper.close(pStmt);
		}
	}
	
	public final static void addCategoryFavorites(Integer userId, Integer categoryId) throws DBException {
		if (userId == null) {
			return;
		}

		Connection conn = null;

		try {
			conn = WebShare.database.getConnection();
			addCategoryFavorites(userId, categoryId, conn);
			conn.commit();
		} catch (Exception e) {
			DBHelper.rollBack(conn);
			throw new DBException(e);
		} finally {
			DBHelper.close(conn);
		}
	}
	
	public final static void addCategoryFavorites(Integer userId, Integer categoryId, Connection conn) throws DBException {
		if ((userId == null || userId.intValue() <= 0) || (categoryId == null || categoryId.intValue() <= 0) ) {
			return;
		}
		
		PreparedStatement pStmt2 = null;
		PreparedStatement pStmt3 = null;
		
		try {
			pStmt2 = conn.prepareStatement("DELETE FROM usercatfavorites WHERE userid = ? AND categoryid = ?");
			pStmt2.setInt(1, userId.intValue());
			pStmt2.setInt(2, categoryId.intValue());
			pStmt2.execute();
			
			pStmt3 = conn.prepareStatement("INSERT INTO usercatfavorites ( userId, categoryId, created ) VALUES ( ?, ?, ? )");
			pStmt3.setInt(1, userId.intValue());
			pStmt3.setInt(2, categoryId.intValue());
			pStmt3.setTimestamp(3, new Timestamp(System.currentTimeMillis()));
			pStmt3.execute();
		} catch (Exception e) {	
			throw new DBException(e);
		} finally {	
			DBHelper.close(pStmt2);
			DBHelper.close(pStmt3);
		}
	}
}