/*
 * $Id: TabViewAction.java,v 1.1 2006/10/04 03:15:03 marbobo Exp $
 *
 * Copyright (c) Siemens AG 2008 
 *           (c) Universitt Paderborn 2008 
 * Licensed under EUPL 1.1. 
 * 
 * $Log: TabViewAction.java,v $
 * Revision 1.1  2006/10/04 03:15:03  marbobo
 * + Vollstndige Untersttzung fr Templates implementiert
 * + Alles von Virtoweb auf ProCoBa umgestellt
 *
 * Revision 1.16  2006/07/05 21:00:08  marbobo
 * + History-Funktionalitt fr Projekt- und Funktionssicht-Tabs implementiert
 *
 * Revision 1.15  2006/07/03 23:46:31  marbobo
 * + Neu implementierung der Projekt- und Funktionssicht-Tabs
 *
 * Revision 1.14  2006/05/31 13:58:40  marbobo
 * + Bugfix
 *
 * Revision 1.13  2006/05/12 22:14:48  marbobo
 * + Administrationssicht implementiert
 *
 * Revision 1.12  2006/05/11 20:25:38  marbobo
 * + Zuletzt besuchte Kategorie-ID, Projekte-ID und Funktionen-ID werden nun gespeichert und beim Sichtenwechsel wiederhergestellt
 *
 * Revision 1.11  2006/03/09 17:17:36  marbobo
 * + Layout-Anpassungen
 *
 * Revision 1.10  2006/02/27 16:15:48  marbobo
 * + Systemsichten hinzugefgt
 * + SysLinks angepasst
 * + Benutzer knnen falls gewnscht Dokumentqueries anlegen
 *
 * Revision 1.9  2004/11/12 00:20:31  marbobo
 * - Unntige Import-Anweisungen entfernt
 *
 * Revision 1.8  2004/11/10 15:02:50  koppy
 * Projektweite berarbeitung von Error-Logs, Stacktraces,..
 *
 * Revision 1.7  2004/11/02 21:42:30  marbobo
 * + Einige Funktionen fr Gast-Benutzer gesperrt
 * + WebShare.LOGIN_PAGE mit WebShare.getLoginPage() ersetzt
 *
 * Revision 1.6  2004/07/20 09:02:07  jonyp
 * added copyright info + cvs log
 *
 *
 */
package vkc.procoba;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import java.util.List;

import vkc.db.serializer.VKCUserFunctionalHistorySerializer;
import vkc.db.serializer.VKCUserProjectHistorySerializer;
import vkc.exception.DBException;
import vkc.misc.VKCLogManager;
import vkc.view.PageId;
import vkc.view.TabId;
import vkc.view.WebShare;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class TabViewAction extends Action {
	private ActionForward DEFAULT_FORWARD = new ActionForward("/templates/${template.name}/procoba/TabView.jsp", false);

	public ActionForward execute(ActionMapping mapping,
								 ActionForm f,
								 HttpServletRequest request,
								 HttpServletResponse response)
	throws ServletException {
		try {
			TabViewForm form = (TabViewForm)f;
			
			if (request.getParameter("action.change.tabview") != null) {
				return handleChangeTabView(form, request);
			} 

			if (request.getParameter("action.update.mostrecenttabview") != null) {
				return handleUpdateMostRecentTabView(form, request);
			} 

			return handleInitializeAction(form, request);
		} catch (Exception e) {
			throw new ServletException(e);
		}
	}

	protected ActionForward handleUpdateMostRecentTabView(TabViewForm form, HttpServletRequest request) throws DBException {
		if (form.isFunctionalViewSelected()) {
			VKCUserFunctionalHistorySerializer.saveFunctionalHistory(form.getUser().getUserId(), form.getSelectedTabId());
		} else if (form.isProjectViewSelected()) {
			VKCUserProjectHistorySerializer.saveProjectHistory(form.getUser().getUserId(), form.getSelectedTabId());
		}
		
		return DEFAULT_FORWARD;
	}

	protected ActionForward handleChangeTabView(TabViewForm form, HttpServletRequest request) {
		request.getSession().setAttribute(WebShare.SYSTEMVIEWTYPEID_KEY, form.getSelectedViewId());

		if (form.getUser().isAllowManageSystemViewType() && PageId.CATEGORY_VIEW.equals(form.getSelectedViewId())) {
			return new ActionForward("/vkcTabView.do?reloadMainview=true&selectedTabId=" + TabId.DOCUMENT_TAB 
									+ "&lastCategoryId=" + form.getLastCategoryId()
									+ "&lastFunctionalId=" + form.getLastFunctionalId()
									+ "&lastProjectId=" + form.getLastProjectId(), true);
		}
		
		if ((form.getUser().hasAdminPrivileges() || form.getUser().isDedicatedAdmin()) && PageId.ADMINISTRATION_VIEW.equals(form.getSelectedViewId())) {
			return new ActionForward("/adminTabView.do?reloadMainview=true&selectedTabId=" + TabId.ADMIN_USERMANAGEMENT_TAB 
									+ "&viewId=" + form.getSelectedViewId()
									+ "&selectedViewId=" + form.getSelectedViewId()
									+ "&lastCategoryId=" + form.getLastCategoryId()
									+ "&lastFunctionalId=" + form.getLastFunctionalId()
									+ "&lastProjectId=" + form.getLastProjectId(), true);
		}
		
		return new ActionForward("/procobaTab.do?reloadMainview=true&viewId=" + form.getSelectedViewId() 
								+ "&selectedViewId=" + form.getSelectedViewId()
								+ "&lastCategoryId=" + form.getLastCategoryId()
								+ "&lastFunctionalId=" + form.getLastFunctionalId()
								+ "&lastProjectId=" + form.getLastProjectId(), true);
	}

	protected ActionForward handleInitializeAction(TabViewForm form, HttpServletRequest request) throws ServletException {
		try {
			String ignoreTabview = request.getParameter("ignoreTabviewLoad");
			
			if (ignoreTabview == null && (form.getSelectedTabId().equals(WebShare.getCategorySettings().getProjectManagementRootId())
			   || form.getSelectedTabId().intValue() <= 0 && form.getSelectedAddTabId().intValue() <= 0)) {
				List tabInfos = form.getTabInfos();
				
				if (tabInfos.size() > 0) {
					if (form.getLastFunctionalId().intValue() > 0 && form.isFunctionalViewSelected()) {
						form.setSelectedTabId(form.getLastFunctionalId());
					} else if (form.getLastProjectId().intValue() > 0 && form.isProjectViewSelected()) {
						form.setSelectedTabId(form.getLastProjectId());
					} else {
						TabInfo tabInfo = (TabInfo)tabInfos.get(0);
						form.setSelectedTabId(tabInfo.getTabId());
					}
				}
			}

			return DEFAULT_FORWARD;
		} catch (Exception e) {
			VKCLogManager.errorLog(this, "", e.getMessage(), e);
			throw new ServletException(e);
		}
	}
}