/*
 * $Id: HiddenTag.java,v 1.2 2005/02/04 18:51:10 jonyp Exp $
 *
 * Copyright (c) Siemens AG 2008 
 *           (c) Universitt Paderborn 2008 
 * Licensed under EUPL 1.1. 
 *
 * $Log: HiddenTag.java,v $
 * Revision 1.2  2005/02/04 18:51:10  jonyp
 * changed EVAL_BODY_TAG (deprecated) to EVAL_PAGE
 *
 * Revision 1.1  2005/02/03 23:26:37  marbobo
 * + Initiale Version
 *
 */

package vkc.view.tag;

import javax.servlet.jsp.JspException;

import org.apache.struts.taglib.html.BaseFieldTag;
import org.apache.struts.taglib.html.Constants;
import org.apache.struts.util.RequestUtils;
import org.apache.struts.util.ResponseUtils;

import vkc.util.VKCStringUtils;

public class HiddenTag extends BaseFieldTag {
    /**
     * The type of input field represented by this tag (text, password, or
     * hidden).
     */
    protected String type = "hidden";

    // ------------------------------------------------------------- Properties
    /**
     * Comma-delimited list of content types that a server processing this form
     * will handle correctly.  This property is defined only for the
     * <code>file</code> tag, but is implemented here because it affects the
     * rendered HTML of the corresponding &lt;input&gt; tag.
     */
    protected String accept = null;

    public String getAccept() {
        return accept;
    }

    public void setAccept( String accept ) {
        this.accept = accept;
    }

    /**
     * The name of the bean containing our underlying property.
     */
    protected String name = Constants.BEAN_KEY;

    public String getName() {
        return name;
    }

    public void setName( String name ) {
        this.name = name;
    }

    /**
     * The "redisplay contents" flag (used only on <code>password</code>).
     */
    protected boolean redisplay = true;

    public boolean getRedisplay() {
        return redisplay;
    }

    public void setRedisplay( boolean redisplay ) {
        this.redisplay = redisplay;
    }


    /**
     * Should the value of this field also be rendered to the response?
     */
    protected boolean write = false;

    public boolean getWrite() {
        return write;
    }

    public void setWrite( boolean write ) {
        this.write = write;
    }

    /**
     * Generate the required input tag, followed by the optional rendered text.
     * Support for <code>write</code> property since Struts 1.1.
     *
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException {
        ResponseUtils.write( this.pageContext, this.renderInputElement() );

        // Is rendering the value separately requested?
        if ( !write ) {
            return EVAL_PAGE;
        }


        // Calculate the value to be rendered separately
        // * @since Struts 1.1
        String results = null;
        
        if ( value != null ) {
            results = VKCStringUtils.filterUnicodeChars( value );
        } else {
            Object value = RequestUtils.lookup( pageContext, name, property, null );
            
            if ( value == null ) {
                results = "";
            } else {
                results = VKCStringUtils.filterUnicodeChars( value.toString() );
            }
        }

        ResponseUtils.write( pageContext, results );
        
        return EVAL_PAGE;
    }

    /**
     * Renders a fully formed &lt;input&gt; element.
     * @throws JspException
     * @since Struts 1.2
     */
    protected String renderInputElement() throws JspException {
        StringBuffer results = new StringBuffer( "<input type=\"" );
        results.append( this.type );
        results.append( "\" name=\"" );

        if ( indexed ) {
            this.prepareIndex( results, name );
        }

        results.append( property );
        results.append( "\"" );
        
        if ( accesskey != null ) {
            results.append( " accesskey=\"" );
            results.append( accesskey );
            results.append( "\"" );
        }

        if ( accept != null ) {
            results.append( " accept=\"" );
            results.append( accept );
            results.append( "\"" );
        }

        if ( maxlength != null ) {
            results.append( " maxlength=\"" );
            results.append( maxlength );
            results.append( "\"" );
        }

        if ( cols != null ) {
            results.append( " size=\"" );
            results.append( cols );
            results.append( "\"" );
        }

        if ( tabindex != null ) {
            results.append( " tabindex=\"" );
            results.append( tabindex );
            results.append( "\"" );
        }

        results.append(" value=\"");
        
        if ( value != null ) {
            results.append( this.formatValue( value ) );

        } else if ( redisplay || !"password".equals( type ) ) {
            Object value =
                RequestUtils.lookup( pageContext, name, property, null );

            results.append( this.formatValue( value ) );
        }

        results.append( '"' );
        results.append( this.prepareEventHandlers() );
        results.append( this.prepareStyles() );
        results.append( this.getElementClose() );

        return results.toString();
    }
    
    /**
     * Return the given value as a formatted <code>String</code>.  This 
     * implementation escapes potentially harmful HTML characters.
     *
     * @param value The value to be formatted. <code>null</code> values will
     * be returned as the empty String "".
     * 
     * @throws JspException if a JSP exception has occurred
     * 
     * @since Struts 1.2
     */
    protected String formatValue( Object value ) {
        if ( value == null ) {
            return "";
        }

        return VKCStringUtils.filterUnicodeChars( value.toString() );
    }

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();
        accept = null;
        name = Constants.BEAN_KEY;
        redisplay = true;
        write = false;
    }
}